
from metakernel.tests.utils import get_kernel, get_log_text


def test_shell_magic():
    kernel = get_kernel()

    text = '%shell ech'
    comp = kernel.do_complete(text, len(text))

    assert 'echo' in comp['matches']

    helpstr = kernel.get_help_on('!cat')
    assert not 'Sorry, no help' in helpstr, helpstr

    helpstr = kernel.get_help_on('%%shell cat', level=1)
    assert not 'Sorry, no help' in helpstr

    helpstr = kernel.get_help_on('!lkjalskdfj')
    assert 'Sorry, no help' in helpstr


def test_shell_magic2():
    kernel = get_kernel()
    kernel.do_execute("!cat \"%s\"" % __file__, False)
    log_text = get_log_text(kernel)
    assert 'metakernel.py' in log_text

    kernel.do_execute('!!\necho "hello"\necho "goodbye"', None)
    log_text = get_log_text(kernel)
    assert '"hello"' in log_text
    assert '"goodbye"' in log_text


def test_shell_magic3():
    kernel = get_kernel()
    kernel.do_execute('!lalkjds')
    text = get_log_text(kernel)
    assert ': command not found' in text, text



# Copyright (c) Metakernel Development Team.
# Distributed under the terms of the Modified BSD License.

from distutils.version import LooseVersion
from metakernel import Magic, option, ExceptionWrapper
import pydoc
import sys
try:
    import jedi
    from jedi import Interpreter
    if jedi.__version__ >= LooseVersion('0.11.0'):
        from jedi.api.helpers import get_on_completion_name
        from parso import split_lines
    elif jedi.__version__ >= LooseVersion('0.10.0'):
        from jedi.api.helpers import get_on_completion_name
        from jedi.common import splitlines as split_lines
    else:
        from jedi.api.helpers import completion_parts
        from jedi.parser.user_context import UserContext
except ImportError:
    jedi = None

PY3 = sys.version_info[0] == 3

def exec_code(code, env, mode):
    import traceback
    try:
        ccode = compile(code, "python cell", mode)
    except Exception as exc:
        ex_type, ex, tb = sys.exc_info()
        tb_format = ["%s: %s" % (ex.__class__.__name__, str(ex))]
        return ExceptionWrapper(ex_type.__name__, repr(exc.args), tb_format)
    try:
        if mode == "exec":
            exec(ccode, env)
        elif mode == "eval":
            return eval(ccode, env)
    except Exception as exc:
        ex_type, ex, tb = sys.exc_info()
        line1 = ["Traceback (most recent call last):"]
        line2 = ["%s: %s" % (ex.__class__.__name__, str(ex))]
        tb_format = line1 + [line.rstrip() for line in traceback.format_tb(tb)[1:]] + line2
        return ExceptionWrapper(ex_type.__name__, repr(exc.args), tb_format)
    if "retval" in env:
        return env["retval"]

class PythonMagic(Magic):

    def __init__(self, kernel):
        super(PythonMagic, self).__init__(kernel)
        self.env = globals()['__builtins__'].copy()
        self.retval = None

    def line_python(self, *args):
        """
        %python CODE - evaluate code as Python

        This line magic will evaluate the CODE (either expression or
        statement) as Python code.

        Note that the version of Python is that of the notebook server.

        Examples:
            %python x = 42
            %python import math
            %python x + math.pi

        """
        code = " ".join(args)
        self.retval = self.eval(code)

    def eval(self, code):
        if "__builtins__" not in self.env:
            ## __builtins__ get generated after an eval:
            eval("1", self.env)
            ## make 'kernel' and 'input' available:
            self.env["__builtins__"]["kernel"] = self.kernel
            if PY3:
                self.env["__builtins__"]["input"] = self.kernel.raw_input
                self.env["input"] = self.kernel.raw_input
            else:
                def input(*args, **kwargs):
                    return eval(self.kernel.raw_input(*args, **kwargs))
                self.env["__builtins__"]["input"] = input
                self.env["input"] = input
                self.env["__builtins__"]["raw_input"] = self.kernel.raw_input = self.kernel.raw_input
                self.env["raw_input"] = self.kernel.raw_input = self.kernel.raw_input
        try:
            ccode = compile(code.strip(), "python cell", "eval") # is it an expression?
        except:
            return exec_code(code.strip(), self.env, "exec") # no, it is statement
        return exec_code(code.strip(), self.env, "eval")     # yes, it is expression

    @option(
        "-e", "--eval_output", action="store_true", default=False,
        help="Use the retval value from the Python cell as code in the kernel language."
    )
    def cell_python(self, eval_output=False):
        """
        %%python - evaluate contents of cell as Python

        This cell magic will evaluate the cell (either expression or
        statement) as Python code.

        Unlike IPython's Python, this does not return the last expression.
        To do that, you need to assign the last expression to the special
        variable "retval".

        The -e or --eval_output flag signals that the retval value expression
        will be used as code for the cell to be evaluated by the host
        language.

        Note that the version of Python is that of the notebook server.

        Examples:
            %%python
            x = 42

            %%python
            import math
            retval = x + math.pi

            %%python -e
            retval = "'(this is code in the kernel language)"

            %%python -e
            "'(this is code in the kernel language)"

        """
        if self.code.strip():
            if eval_output:
                self.eval(self.code)
                self.code = str(self.env["retval"]) if ("retval" in self.env and
                                                        self.env["retval"] != None) else ""
                self.retval = None
                self.env["retval"] = None
                self.evaluate = True
            else:
                self.retval = self.eval(self.code)
                self.env["retval"] = None
                self.evaluate = False

    def post_process(self, retval):
        if retval is not None:
            return retval
        else:
            return self.retval

    def get_completions(self, info):
        '''Get Python completions'''
        # https://github.com/davidhalter/jedi/blob/master/jedi/utils.py
        if jedi is None:
            return []

        text = info['code']
        position = (info['line_num'], info['column'])
        interpreter = Interpreter(text, [self.env])

        if jedi.__version__ >= LooseVersion('0.12.0'):
            lines = split_lines(text)
            name = get_on_completion_name(
                interpreter._module_node,
                lines,
                position
            )
            before = text[:len(text) - len(name)]
        elif jedi.__version__ >= LooseVersion('0.10.0'):
            lines = split_lines(text)
            name = get_on_completion_name(
                interpreter._get_module_node(),
                lines,
                position
            )
            before = text[:len(text) - len(name)]
        else:
            path = UserContext(text, position).get_path_until_cursor()
            path, dot, like = completion_parts(path)
            before = text[:len(text) - len(like)]

        completions = interpreter.completions()
        completions = [before + c.name_with_symbols for c in completions]

        self.kernel.log.error(completions)

        return [c[info['start']:] for c in completions]

    def get_help_on(self, info, level=0, none_on_fail=False):
        """Implement basic help for functions"""
        if not info['code']:
            return None if none_on_fail else ''

        last = info['obj']

        default = None if none_on_fail else ('No help available for "%s"' % last)

        parts = last.split('.')

        obj = self.env.get(parts[0], None)

        if not obj:
            return default

        for p in parts[1:]:

            obj = getattr(obj, p, None)

            if not obj:
                return default

        strhelp = pydoc.render_doc(obj, "Help on %s")
        if level == 0:
            return getattr(obj, '__doc__', strhelp)
        else:
            return strhelp


def register_magics(kernel):
    kernel.register_magics(PythonMagic)




import os
from metakernel.tests.utils import get_kernel


def test_file_magic():
    kernel = get_kernel()
    kernel.do_execute("""%%file TEST.txt
LINE1
LINE2
LINE3""", False)
    assert os.path.exists("TEST.txt")
    with open("TEST.txt") as fp:
        lines = fp.readlines()
        assert len(lines) == 3
        assert lines[0] == "LINE1\n"
        assert lines[1] == "LINE2\n"
        assert lines[2] == "LINE3"

    kernel.do_execute("""%%file -a TEST.txt

LINE4
LINE5
LINE6""", False)
    assert os.path.exists("TEST.txt")
    with open("TEST.txt") as fp:
        lines = fp.readlines()
        assert len(lines) == 6
        assert lines[3] == "LINE4\n"
        assert lines[4] == "LINE5\n"
        assert lines[5] == "LINE6"

    kernel.do_execute("""%%file /tmp/tmp/TEST.txt
TEST1
TEST2
TEST3""")
    with open("/tmp/tmp/TEST.txt") as fp:
        lines = fp.readlines()
        assert len(lines) == 3
        assert lines[0] == "TEST1\n"
        assert lines[1] == "TEST2\n"
        assert lines[2] == "TEST3"

def teardown():
    import shutil
    shutil.rmtree("/tmp/tmp", ignore_errors=True)



"""
==========
tutormagic
==========

Magics to display pythontutor.com in the notebook.
"""

# Based on Kiko Correoso's IPython magic:
# https://github.com/kikocorreoso/tutormagic
# and Doug Blank's
# http://jupyter.cs.brynmawr.edu/hub/dblank/public/Examples/Online%20Python%20Tutor.ipynb
#-----------------------------------------------------------------------------
# Copyright (C) 2015 Kiko Correoso and the pythontutor.com developers
#
# Distributed under the terms of the MIT License. The full license is in
# the file LICENSE, distributed as part of this software.
#
# Contributors:
#   kikocorreoso
#-----------------------------------------------------------------------------

from metakernel import Magic, option
from IPython.display import IFrame
import sys
if sys.version_info.major == 2 and sys.version_info.minor == 7:
    from urllib import quote
elif sys.version_info.major == 3 and sys.version_info.minor >= 3:
    from urllib.parse import quote
    
class TutorMagic(Magic):

    @option(
        '-l', '--language', action='store', nargs = 1,
        help=("Possible languages to be displayed within the iframe. " +
              "Possible values are: python, python2, python3, java, javascript")
    )
    def cell_tutor(self, language=None):
        """
        %%tutor [--language=LANGUAGE] - show cell with 
        Online Python Tutor.

        Defaults to use the language of the current kernel.
        'python' is an alias for 'python3'.

        Examples:
           %%tutor -l python3
           a = 1
           b = 1
           a + b

           [You will see an iframe with the pythontutor.com page 
           including the code above.]

           %%tutor --language=java
           
           public class Test {
               public Test() {
               }
               public static void main(String[] args) {
                   int x = 1;
                   System.out.println("Hi");
               }
           }
        """
        if language is None:
            language = self.kernel.language_info["name"]
        if language not in ['python', 'python2', 'python3', 'java', 'javascript']:
            raise ValueError("{} not supported. Only the following options are allowed: "
                             "'python2', 'python3', 'java', 'javascript'".format(language))
        
        url = "https://pythontutor.com/iframe-embed.html#code="
        url += quote(self.code)
        url += "&origin=opt-frontend.js&cumulative=false&heapPrimitives=false"
        url += "&textReferences=false&"
        if language in ["python3", "python"]:
            url += "py=3&rawInputLstJSON=%5B%5D&curInstr=0&codeDivWidth=350&codeDivHeight=400"
        elif language == "python2":
            url += "py=2&rawInputLstJSON=%5B%5D&curInstr=0&codeDivWidth=350&codeDivHeight=400"
        elif language == "java":
            url += "py=java&rawInputLstJSON=%5B%5D&curInstr=0&codeDivWidth=350&codeDivHeight=400"
        elif language == "javascript":
            url += "py=js&rawInputLstJSON=%5B%5D&curInstr=0&codeDivWidth=350&codeDivHeight=400"

        # Display the results in the output area
        self.kernel.Display(IFrame(url, height=500, width="100%"))
        self.evaluate = False

def register_magics(kernel):
    kernel.register_magics(TutorMagic)

def register_ipython_magics():
    from metakernel import IPythonKernel
    from IPython.core.magic import register_cell_magic
    kernel = IPythonKernel()
    magic = TutorMagic(kernel)

    @register_cell_magic
    def tutor(line, cell):
        magic.code = cell
        magic.cell_tutor(language="python3")



from IPython.paths import get_ipython_dir
import os


def get_history_file(kernel):
    """Gets the history file for the kernel.

    Histories are stored in ~/.ipython/metakernel/history
    """
    base = get_ipython_dir()
    dname = os.path.join(base, 'metakernel', 'history')
    if not os.path.exists(dname):
        os.makedirs(dname)
    if hasattr(kernel, 'implementation'):
        fname = kernel.implementation.lower()
    else:
        fname = kernel.__class__.__name__
        fname = fname.replace("Magic", '').lower()
    path = os.path.join(dname, fname + '.json')
    with open(path, 'a'):
        pass
    return path


def get_local_magics_dir():
    """
    Ensures that there is a ~/.ipython/metakernel/magics directory,
    and returns the path to it.
    """
    base = get_ipython_dir()
    dname = os.path.join(base, 'metakernel', 'magics')
    if not os.path.exists(dname):
        os.makedirs(dname)
    return dname




from metakernel.tests.utils import (get_kernel, get_log_text, EvalKernel,
                                    clear_log_text)


def test_macro_magic():
    kernel = get_kernel(EvalKernel)
    kernel.do_execute("""%%macro testme
print("ok")
    """, False)
    kernel.do_execute("%macro testme", False)
    text = get_log_text(kernel)
    assert "ok" in text, text
    clear_log_text(kernel)

    kernel.do_execute("%macro -l learned", False)
    text = get_log_text(kernel)
    assert "testme" in text, text
    clear_log_text(kernel)

    kernel.do_execute("%macro -d testme", False)
    kernel.do_execute("%macro -l learned", False)
    text = get_log_text(kernel)
    assert "testme" not in text, text
    clear_log_text(kernel)

    




from IPython.display import HTML

from metakernel.tests.utils import get_kernel, get_log_text
from metakernel.process_metakernel import BashKernel


def test_process_metakernel():
    kernel = get_kernel(BashKernel)
    kernel.do_execute("cat \"%s\"" % __file__, False)
    log_text = get_log_text(kernel)
    assert 'metakernel.py' in log_text, log_text

    kernel.do_execute('echo "hello"\necho "goodbye"', None)
    log_text = get_log_text(kernel)
    assert '"hello"' in log_text
    assert '"goodbye"' in log_text

    kernel.do_execute('lalkjds')
    text = get_log_text(kernel)
    assert ': command not found' in text, text
    
    html = HTML("some html")
    kernel.Display(html)

    kernel.do_execute(r'for i in {1..3};do echo -ne "$i\r"; sleep 1; done', None)
    text = get_log_text(kernel)
    assert r'1\r2\r3\r' in text


# Copyright (c) Metakernel Development Team.
# Distributed under the terms of the Modified BSD License.

from metakernel import Magic

class GetMagic(Magic):
    def line_get(self, variable):
        """
        %get VARIABLE - get a variable from the kernel in a Python-type.

        This line magic is used to get a variable.

        Examples:
            %get x 
        """
        self.retval = self.kernel.get_variable(variable)

    def post_process(self, retval):
        return self.retval


def register_magics(kernel):
   kernel.register_magics(GetMagic)



# Copyright (c) Metakernel Development Team.
# Distributed under the terms of the Modified BSD License.

from metakernel import Magic, option

class MatplotlibMagic(Magic):
    """
    Magic for using matplotlib with kernels other than ipython.
    """
    def line_matplotlib(self, backend):
        """
        %matplotlib BACKEND - set the matplotlib backend to BACKEND

        This line magic will set (and reload) the items associated
        with the matplotlib backend. 

        Also, monkeypatches the IPython.display.display
        to work with metakernel-based kernels.

        Example:
            %matplotlib notebook

            import matplotlib.pyplot as plt
            plt.plot([3, 8, 2, 5, 1])
            plt.show()
        """
        import IPython.display
        import metakernel.display
        import imp

        # Monkeypatch IPython.display.display:
        IPython.display.display = metakernel.display.display

        import matplotlib
        imp.reload(matplotlib)

        if backend == "notebook":
            backend = "nbagg"

        matplotlib.use(backend)

        import matplotlib.backends.backend_nbagg
        imp.reload(matplotlib.backends.backend_nbagg)

        import matplotlib.backends.backend_webagg_core
        imp.reload(matplotlib.backends.backend_webagg_core)

def register_magics(kernel):
    kernel.register_magics(MatplotlibMagic)




from metakernel.tests.utils import get_kernel


def test_plot_magic_backend():
    kernel = get_kernel()
    kernel.do_execute('%plot qt -f svg -s400,200', None)
    assert kernel.plot_settings['width'] == 400
    assert kernel.plot_settings['height'] == 200
    assert kernel.plot_settings['format'] == 'svg'
    assert kernel.plot_settings['backend'] == 'qt'

def test_plot_magic_format():
    kernel = get_kernel()
    kernel.do_execute('%plot qt -f svg -w 500 -h 400 -r 200')
    assert kernel.plot_settings['backend'] == 'qt', kernel.plot_settings
    assert kernel.plot_settings['format'] == 'svg', kernel.plot_settings
    assert kernel.plot_settings['width'] == 500, kernel.plot_settings
    assert kernel.plot_settings['height'] == 400, kernel.plot_settings
    assert kernel.plot_settings['resolution'] == 200, kernel.plot_settings

def test_plot_magic_size():
    kernel = get_kernel()
    kernel.do_execute('%plot qt4 -s 400,200')
    assert kernel.plot_settings['width'] == 400
    assert kernel.plot_settings['height'] == 200
    assert kernel.plot_settings['backend'] == 'qt4', kernel.plot_settings

def test_plot_magic_all():
    kernel = get_kernel()
    kernel.do_execute('%plot -b qt5 -f svg -s 400,200')
    assert kernel.plot_settings['width'] == 400
    assert kernel.plot_settings['height'] == 200
    assert kernel.plot_settings['format'] == 'svg', kernel.plot_settings
    assert kernel.plot_settings['backend'] == 'qt5', kernel.plot_settings




# Copyright (c) Metakernel Development Team.
# Distributed under the terms of the Modified BSD License.

from metakernel import Magic, option
from IPython.display import HTML, Javascript
import string
import random
import os

try:
    import urllib.request
    urlopen = urllib.request.urlopen
except: # python2
    import urllib
    urlopen = urllib.urlopen

def download(url):
    g = urlopen(url)
    return g.read().decode("utf-8")

class JigsawMagic(Magic):

    @option(
        '-w', '--workspace', action='store', default=None,
        help='use the provided name as workspace filename'
    )
    def line_jigsaw(self, language, workspace=None):
        """
        %jigsaw LANGUAGE - show visual code editor/generator

        This line magic will allow visual code editing or generation.

        Examples:
            %jigsaw Processing
            %jigsaw Python
            %jigsaw Processing --workspace workspace1
        """
        # Copy iframe html to here (must come from same domain):
        # Make up a random workspace name:
        if workspace is None:
            workspace = "jigsaw-workspace-" + (''.join(random.SystemRandom().choice(string.ascii_uppercase + string.digits) for i in range(6)))
        workspace_filename = workspace + ".xml"
        html_text = download("https://calysto.github.io/jigsaw/" + language + ".html")
        html_filename = workspace + ".html"
        html_text = html_text.replace("MYWORKSPACENAME", workspace_filename)
        with open(html_filename, "w") as fp:
            fp.write(html_text)
        # Display iframe:
        script = """
    if (document.jigsaw_register_workspace === undefined) {

        document.jigsaw_workspaces = {};

        document.jigsaw_register_workspace = function(workspace_filename, workspace, xml_init) {
            workspace.element = document.element;
            document.jigsaw_workspaces[workspace_filename] = workspace;

            try {
                $([window.parent.IPython.events]).on('notebook_saved.Notebook', function() { 
                    try {
                        document.jigsaw_save_workspace(workspace_filename); 
                    } catch(err) {
                        // ignore failure, might not exist
                    }
                });
            } catch (err) {
                // rendering for display
            }
            
            var xml = document.jigsaw_loadXMLDoc(workspace_filename);
            if (xml === null) {
                xml = xml_init;
                if (xml === null) {
                    xml = Blockly.Xml.textToDom('<xml id="workspace"></xml>');
                }
            } else {
                xml = xml.children[0]; // document
            }
            Blockly.Xml.domToWorkspace(workspace, xml);
        };

        document.jigsaw_handle_output = function(workspace_filename, out) {
            var workspace = document.jigsaw_workspaces[workspace_filename];
            //var output_area = workspace.element.output_area;
            var cell_index = document.jigsaw_get_cell(workspace.element);
            var cell = IPython.notebook.get_cell(cell_index);
            var res = null;
            var data = null;
            document.cell = cell;
            document.out = out;
            if (out.msg_type == "stream") {
                res = out.content.text;
                //document.getElementById('code_output').value += res.toString();
            } else if (out.msg_type === "pyout") {
                // if output is a python object
                res = out.content.data["text/plain"];
                //document.getElementById('code_output').value += res.toString(); 
            } else if (out.msg_type == "pyerr") {
                // if output is a python error
                res = out.content.data["text/plain"];
                //document.getElementById('code_output').value += res.toString();
            } else if (out.msg_type == "execute_result") {
                var str = out.content.data["text/plain"];
                res = str;
                if (res.indexOf("u") == 0)
                    res = res.substring(2, res.length - 1) + "\\n";
                if (res) {
                    //document.getElementById('code_output').value += res.toString();
                }
            } else if (out.msg_type == "error") {
                res = out.content.ename + ": " + out.content.evalue + "\\n";
                // FIXME: out.traceback is Array of terminal color-coded [-codes
            } else {
                // if output is something we haven't thought of
                res = out.toString();
                //document.getElementById('code_output').value += res.toString();
            }
            if (res) {
                cell.output_area.append_output({output_type: "stream", text: res.toString(), name: "output"});
            }
        };
        
        document.jigsaw_generate = function(workspace_filename, language, insert_code) {
            var workspace = document.jigsaw_workspaces[workspace_filename];
            var callbacks = { 'iopub' : {'output' : function(out) { document.jigsaw_handle_output(workspace_filename, out); }}};
            var code = Blockly[language].workspaceToCode(workspace);
            if (insert_code == 1) {
                var cell_index = document.jigsaw_get_cell(workspace.element);
                var cell = IPython.notebook.insert_cell_at_index(0, cell_index + 1);
                cell.set_text(code);
            } else {
                window.parent.IPython.notebook.kernel.execute(code,
                                                              callbacks,
                                                              {silent: false});
            }
        };
        
        document.jigsaw_save_workspace = function(workspace_filename) {
            var workspace = document.jigsaw_workspaces[workspace_filename];
            var xml = Blockly.Xml.workspaceToDom(workspace);
            document.xml = xml;
            if (xml !== undefined) {
                console.log(xml);
                //xml.style = "display: none";
                //xml.id = "workspace";
                var xml_text = Blockly.Xml.domToText(xml)
                IPython.notebook.kernel.execute('%%file ' + workspace_filename + '\\n' + xml_text);
            }
        };
        
        document.jigsaw_loadXMLDoc = function(filename) {
            var xhttp = new XMLHttpRequest();
            xhttp.open("GET", filename, false);
            xhttp.send();
            return xhttp.responseXML;
        };
    }

    document.jigsaw_get_cell = function (element) {
        // FIXME: brittle and ugly:
        var mydiv = element[0].parentNode.parentNode.parentNode.parentNode;
        var cells = IPython.notebook.get_cells();
        for (var i = 0; i < cells.length; i++) {
            if (mydiv === cells[i].element[0]) {
                return i;
            }
        }
        return null;
    };

    document.jigsaw_clear_output = function (workspace_filename) {
        var workspace = document.jigsaw_workspaces[workspace_filename];
        var cell_index = document.jigsaw_get_cell(workspace.element);
        var cell = IPython.notebook.get_cell(cell_index);
        // FIXME: brittle and ugly:
        cell.element[0].children[2].children[1].children[2].children[1].children[0].innerHTML = ""
        cell.output_area.outputs[2].text = ""
    };

    try {
        document.element = element;
    } catch (err) {
        // rendering
    }
"""
        script = script.replace("MYWORKSPACENAME", workspace_filename);
        iframe = """<iframe src="%s" width="100%%" height="350" style="resize: both; overflow: auto;"></iframe>""" % html_filename
        self.kernel.Display(Javascript(script))
        self.kernel.Display(HTML(iframe))

def register_magics(kernel):
    kernel.register_magics(JigsawMagic)

def register_ipython_magics():
    from metakernel import IPythonKernel
    from IPython.core.magic import register_line_magic
    kernel = IPythonKernel()
    magic = JigsawMagic(kernel)
    # Make magics callable:
    kernel.line_magics["jigsaw"] = magic

    @register_line_magic
    def jigsaw(line):
        """
        Use the Jigsaw code visualizer and generator.
        """
        kernel.call_magic("%jigsaw " + line)



# Copyright (c) Metakernel Development Team.
# Distributed under the terms of the Modified BSD License.


from metakernel import Magic, option
import os

class EditMagic(Magic):

    def line_edit(self, filename):
        """
        %edit FILENAME - load code from filename into next cell for editing

        This line magic will open the file in the next cell, and allow
        you edit it.

        This is a shortcut for %load, and appending a "%%file" as first line.

        Example:
            %edit myprogram.ss

        """
        orig_filename = filename
        if filename.startswith("~"):
            filename = os.path.expanduser(filename)
        filename = os.path.abspath(filename)
        text = open(filename).read()
        self.kernel.payload.append({"source": "set_next_input",
                                    "text": "%%file " + orig_filename + "\n" + text})

def register_magics(kernel):
    kernel.register_magics(EditMagic)




from metakernel.tests.utils import (get_kernel, get_log_text, EvalKernel,
                                    clear_log_text)


def test_pipe_magic():
    kernel = get_kernel(EvalKernel)
    kernel.do_execute("""

def upper(text):
    return text.upper()

def lower(text):
    return text.lower()

def pl_word(word):
    if len(word) > 3:
        return word[1:] + word[0] + "ay"
    else:
        return word

def piglatin(text):
    return " ".join([pl_word(word) for word in text.split(" ")])

""")
    kernel.do_execute("""%%pipe upper
this is a test
 """)
    text = get_log_text(kernel)
    assert "THIS IS A TEST" in text, ("text: " + text)

    kernel.do_execute("""%%pipe upper | piglatin
this is a test
 """)
    text = get_log_text(kernel)
    assert "HISTay IS A ESTTay" in text, ("text: " + text)

    kernel.do_execute("""%%pipe piglatin | upper
this is a test
 """)
    text = get_log_text(kernel)
    assert "HISTAY IS A ESTTAY" in text, ("text: " + text)

    kernel.do_execute("""%%pipe piglatin | upper | lower
this is a test
 """)
    text = get_log_text(kernel)
    assert "histay is a esttay" in text, ("text: " + text)
    clear_log_text(kernel)






from metakernel.tests.utils import (get_kernel, get_log_text, 
                                    clear_log_text, EvalKernel)

def test_edit_magic():
    kernel = get_kernel(EvalKernel)

    results = kernel.do_execute("%%edit %s" % __file__)
    text = results["payload"][0]["text"]
    assert text.startswith('%%file')
    assert 'def test_edit_magic' in text



# Copyright (c) Metakernel Development Team.
# Distributed under the terms of the Modified BSD License.

from metakernel import Magic
import json

class ConnectInfoMagic(Magic):

    def line_connect_info(self, dummy=None):
        """
        %connect_info - show connection information

        This line magic will show the connection information for this
        language kernel instance. This information is only necessary
        if you are interested in making additional connections to the
        running kernel.

        Example:
            %connect_info

        Paste the given JSON into a file, and connect with:

            $> ipython <app> --existing <file>

        or, if you are local, you can connect with just:

            $> ipython <app> --existing %(key)s

        or even just:
            $> ipython <app> --existing

        if this is the most recent Jupyter session you have started.
        """
        connection_file = self.kernel.config["IPKernelApp"]["connection_file"]
        try:
            config = json.loads(open(connection_file).read())
        except:
            config = {"stdin_port": "UNKNOWN",
                      "shell_port": "UNKNOWN",
                      "iopub_port": "UNKNOWN",
                      "hb_port": "UNKNOWN",
                      "ip": "UNKNOWN",
                      "key": "UNKNOWN",
                      "signature_scheme": "UNKNOWN",
                      "transport": "UNKNOWN"
            }
        retval = """{
  "stdin_port": %(stdin_port)s,
  "shell_port": %(shell_port)s,
  "iopub_port": %(iopub_port)s,
  "hb_port": %(hb_port)s,
  "ip": "%(ip)s",
  "key": "%(key)s",
  "signature_scheme": "%(signature_scheme)s",
  "transport": "%(transport)s"
}

Paste the above JSON into a file, and connect with:
    $> ipython <app> --existing <file>
or, if you are local, you can connect with just:
    $> ipython <app> --existing %(key)s

or even just:
    $> ipython <app> --existing
if this is the most recent Jupyter session you have started.
""" % config
        self.kernel.Print(retval)

def register_magics(kernel):
    kernel.register_magics(ConnectInfoMagic)



# Copyright (c) Metakernel Development Team.
# Distributed under the terms of the Modified BSD License.

from metakernel import Magic, option
import logging
import time

class Slice(object):
    """Utility class for making slice ranges."""
    def __getitem__(self, item):
        return item

slicer = Slice() ## instance to parse slices

class ParallelMagic(Magic):
    client = None
    view = None
    view_load_balanced = None
    module_name = None
    class_name = None
    kernel_name = None
    ids = None
    retval = None
    retry = False

    @option(
        '-k', '--kernel_name', action='store', default="default",
        help='arbitrary name given to reference kernel'
    )
    @option(
        '-i', '--ids', action='store', default=None,
        help='the machine ids to use from the cluster'
        
    )
    def line_parallel(self, module_name, class_name, kernel_name="default", ids=None):
        """
        %parallel MODULE CLASS [-k NAME] [-i [...]] - construct an interface to the cluster.

        Example:

            %parallel bash_kernel BashKernel 
            %parallel bash_kernel BashKernel -k bash
            %parallel bash_kernel BashKernel -i [0,2:5,9,...]

        cluster_size and cluster_rank variables are set upon 
        initialization of the remote node (if the kernel 
        supports %set).

        Use %px or %%px to send code to the cluster.
        """
        try:
            from ipyparallel import Client
        except ImportError:
            from IPython.parallel import Client
        count = 1
        while count <= 5:
            try:
                self.client = Client()
                break
            except:
                print("Waiting on cluster to start...")
                time.sleep(2)
            count += 1
        if count == 6:
            raise Exception("Cluster was not started.")
        if ids is None:
            count = 1
            while count <= 5:
                try:
                    self.view = self.client[:]
                    break
                except:
                    print("Waiting for engines...")
                    time.sleep(2)
                count += 1
            if count == 6:
                raise Exception("Engines were not started.")
        else:
            # ids[:] = slice(None, None, None)
            # ids[1:3] = slice(1, 3, None)
            # ids[1:3:1] = slice(1, 3, 1)
            # ids[1, 2, ...] = [1, 2, Ellipsis]
            # ids[1, 2:4, ...] = [1, slice(2, 4, None), Ellipsis]
            try:
                ids_slice = eval("slicer%s" % ids) # slicer[0,...,7]
            except:
                ids_slice = slicer[:]
            if isinstance(ids_slice, (slice, int)):
                count = 1
                while count <= 5:
                    try:
                        self.view = self.client[ids_slice]
                        break
                    except:
                        print("Waiting for engines...")
                        time.sleep(2)
                    count += 1
                if count == 6:
                    raise Exception("Engines were not started.")
            else: # tuple of indexes/slices
                # FIXME: if so, handle Ellipsis
                view = None
                for item in ids_slice:
                    count = 1
                    while count <= 5:
                        try:
                            client = self.client[item]
                            if view:
                                ## FIXME: can't do this:
                                view.append(client)
                            else:
                                view = client
                            break
                        except:
                            print("Waiting on cluster to start...")
                            time.sleep(2)
                        count += 1
                    if count == 6:
                        raise Exception("Cluster was not started.")
                self.view = view
        self.view_load_balanced = self.client.load_balanced_view()
        self.module_name = module_name
        self.class_name = class_name
        self.kernel_name = kernel_name
        self.view.execute("""
import os
for key, value in %(env)s.items():
    os.environ[key] = value
try:
    kernels
except:
    kernels = {}
from %(module_name)s import %(class_name)s
kernels['%(kernel_name)s'] = %(class_name)s()
## FIXME: kernels['%(kernel_name)s'].kernel = kernel
""" % {"module_name": module_name, 
       "class_name": class_name,
       "kernel_name": kernel_name,
       "env": str(self.kernel.env)}, 
                          block=True)

        self.view["kernels['%s'].set_variable(\"cluster_size\", %s)" % (
            kernel_name, len(self.client))]
        self.client[:].scatter('cluster_rank', self.client.ids, flatten=True)
        self.view["kernels['%s'].set_variable(\"cluster_rank\", cluster_rank)" % (
            kernel_name)]
        ## So that these are available in the host kernel:
        self.kernel.set_variable("cluster_size", len(self.client))
        self.kernel.set_variable("cluster_rank", -1)
        self.retval = None

    @option(
        '-k', '--kernel_name', action='store', default=None,
        help='kernel name given to use for execution'
    )
    @option(
        '-e', '--evaluate', action='store_true', default=False,
        help=('evaluate code in the current kernel, too. The current ' +
              'kernel should be of the same language as the cluster.')
    )
    @option(
        '-s', '--set_variable', action='store', default=None,
        help='set the variable with the parallel results rather than returning them'
    )
    def line_px(self, expression, kernel_name=None, evaluate=False, set_variable=None):
        """
        %px EXPRESSION - send EXPRESSION to the cluster.

        Example:

            %px sys.version
            %px -k scheme (define x 42)
            %px x
            %px cluster_rank

        cluster_size and cluster_rank variables are set upon 
        initialization of the remote node (if the kernel 
        supports %set).

        Use %parallel to initialize the cluster.
        """
        results = None
        expression = str(expression)
        if kernel_name is None:
            kernel_name = self.kernel_name
        if self.retry:
            count = 1
            while count <= 5:
                try:
                    results = self.view["kernels['%s'].do_execute_direct(\"%s\")" % (
                        kernel_name, self._clean_code(expression))]
                    break
                except:
                    print("Waiting on cluster clients to start...")
                    time.sleep(2)
                count += 1
            if count == 6:
                raise Exception("Cluster clients have not started.")
            self.retry = False
        else:
            try:
                results = self.view["kernels['%s'].do_execute_direct(\"%s\")" % (
                    kernel_name, self._clean_code(expression))]
            except Exception as e:
                results = str(e)
        if set_variable is None:
            self.retval = results
        else:
            self.kernel.set_variable(set_variable, results)
            self.retval = None
        if evaluate:
            self.code = expression

    def _clean_code(self, expr):
        return expr.strip().replace('"', '\\"').replace("\n", "\\n")

    ## px --kernel NAME
    @option(
        '-k', '--kernel_name', action='store', default=None,
        help='kernel name given to use for execution'
    )
    @option(
        '-e', '--evaluate', action='store_true', default=False,
        help=('evaluate code in the current kernel, too. The current ' +
              'kernel should be of the same language as the cluster.')
    )
    @option(
        '-s', '--set_variable', action='store', default=None,
        help='set the variable with the parallel results rather than returning them'
    )
    def cell_px(self, kernel_name=None, evaluate=False, set_variable=None):
        """
        %%px - send cell to the cluster.

        Example:

            %%px 
            (define x 42)

        Use %parallel to initialize the cluster.
        """
        if kernel_name is None:
            kernel_name = self.kernel_name
        results = self.view["kernels['%s'].do_execute_direct(\"%s\")" % (
            kernel_name, self._clean_code(self.code))]
        if set_variable is None:
            self.retval = results
        else:
            self.kernel.set_variable(set_variable, results)
            self.retval = None
        self.evaluate = evaluate

    @option(
        '-s', '--set_variable', action='store', default=None,
        help='set the variable with the parallel results rather than returning them'
    )
    def line_pmap(self, function_name, args, kernel_name=None, set_variable=None):
        """
        %pmap FUNCTION [ARGS1,ARGS2,...] - ("parallel map") call a FUNCTION on args

        This line magic will apply a function name to all of the
        arguments given one at a time using a dynamic load balancing scheduler.

        Currently, the args are provided as a Python expression (with no spaces).

        You must first setup a cluster using the %parallel magic.

        Examples:

            %pmap function-name-in-language range(10)
            %pmap function-name-in-language [1,2,3,4]
            %pmap run_experiment range(1,100,5)
            %pmap run_experiment ["test1","test2","test3"]
            %pmap f [(1,4,7),(2,3,5),(7,2,2)]

        The function name must be a function that is available on all
        nodes in the cluster. For example, you could:

            %%px
            (define myfunc
               (lambda (n)
                 (+ n 1)))

        to define myfunc on all machines (use %%px -e to also define
        it in the running notebook or console). Then you can apply it
        to a list of arguments:

            %%pmap myfunc range(100)

        The load balancer will run myfunc on the next available node
        in the cluster.

        Note: not all languages may support running a function via this magic.
        """
        if kernel_name is None:
            kernel_name = self.kernel_name

        # To make sure we can find `kernels`:
        try:
            from ipyparallel.util import interactive
        except ImportError:
            from IPython.parallel.util import interactive
        f = interactive(lambda arg, kname=kernel_name, fname=function_name: \
                        kernels[kname].do_function_direct(fname, arg))
        results = self.view_load_balanced.map_async(f, eval(args))
        if set_variable is None:
            self.retval = results
        else:
            self.kernel.set_variable(set_variable, results)
            self.retval = None

    def post_process(self, retval):
        try:
            ## any will crash on numpy arrays
            if isinstance(self.retval, list) and not any(self.retval):
                return None
        except:
            pass
        return self.retval

def register_magics(kernel):
    kernel.register_magics(ParallelMagic)




from IPython.display import *

ipdisplay = display
ipclear_output = clear_output

def display(*args, **kwargs):
    from . import get_metakernel
    kernel = get_metakernel()
    if kernel:
        kernel.Display(*args, **kwargs)
    else:
        ipdisplay(*args, **kwargs)

def clear_output(*args, **kwargs):
    from . import get_metakernel
    kernel = get_metakernel()
    if kernel:
        kernel.clear_output(*args, **kwargs)
    else:
        ipclear_output(*args, **kwargs)



# Copyright (c) Metakernel Development Team.
# Distributed under the terms of the Modified BSD License.

from metakernel import Magic

class SetMagic(Magic):
    def line_set(self, variable, value):
        """
        %set VARIABLE VALUE - set a variable in the kernel.

        This line magic is used to set a variable to a Python value.

        Examples:
            %set x 42
            %set x [1, 2, 3]
        """
        value = self.kernel.do_execute_direct(value)
        self.kernel.set_variable(variable, value)

    def post_process(self, retval):
        return retval


def register_magics(kernel):
   kernel.register_magics(SetMagic)




from metakernel.tests.utils import get_kernel, get_log_text, EvalKernel
import os
import time

def test_kernel_magic():
    kernel = get_kernel(EvalKernel)
    kernel.do_execute("%kx 42", False)
    results = get_log_text(kernel)
    assert "42" in results, results
    



from metakernel import Magic, option
from metakernel.tests.utils import get_kernel, get_log_text


class Dummy(Magic):

        @option(
        '-s', '--size', action='store',
        help='Pixel size of plots, "width,height"'
         )
        def line_dummy(self, foo, size=None):
            """
            %dummy [options] foo - Perform dummy operation on foo

            This is additional information on dummy.

            """
            self.foo = foo
            self.size = size

        def cell_spam(self):
            """
            %%spam - Cook some spam
            """
            pass

        def line_eggs(self, style):
            """
            %eggs STYLE - cook some eggs in the given style
            """
            pass


def test_get_magics():
    kernel = get_kernel()
    d = Dummy(kernel)
    line = d.get_magics('line')
    cell = d.get_magics('cell')

    assert 'dummy' in line
    assert 'spam' in cell
    assert 'eggs' in line


def test_get_help():
    kernel = get_kernel()
    d = Dummy(kernel)

    dummy_help = d.get_help('line', 'dummy', 0)
    assert dummy_help.startswith("%dummy")
    assert "    This is additional information on dummy" in d.line_dummy.__doc__, "Checking indents"
    assert "\nThis is additional information on dummy" in dummy_help, "Checking indent removal"

    dummy_help = d.get_help('line', 'dummy', 1)
    # will show this entire file, including this sentence
    assert "# will show this entire file, including this sentence" in \
            dummy_help

    spam_help = d.get_help('cell', 'spam', 0)
    assert spam_help.startswith("%%spam"), spam_help

    spam_help = d.get_help('cell', 'spam', 1)
    assert "# will show this entire file, including this sentence" in \
            spam_help


def test_option():
    kernel = get_kernel()
    d = Dummy(kernel)
    assert 'Options:' in d.line_dummy.__doc__
    assert '--size' in d.line_dummy.__doc__

    ret = d.call_magic('line', 'dummy', '', 'hey -s400,200')
    assert ret == d
    assert d.foo == 'hey', d.foo
    assert d.size == (400, 200)

    ret = d.call_magic('line', 'dummy', '', 'hey there')
    assert d.foo == 'hey there'

    ret = d.call_magic('line', 'dummy', '', 'range(1, 10)')
    # arg eval no longer evals Python raw code:
    assert d.foo == "range(1, 10)"

    ret = d.call_magic('line', 'dummy', '', '[1, 2, 3]')
    # arg eval does eval Python data structures:
    assert d.foo == [1, 2, 3]

    ret = d.call_magic('line', 'dummy', '', 'hey -l -s400,200')
    assert d.size == (400, 200)
    assert d.foo == "hey -l"

    ret = d.call_magic('line', 'dummy', '', 'hey -s -- -s400,200')
    assert d.size == (400, 200)
    assert d.foo == "hey -s"




from metakernel.tests.utils import (get_kernel, get_log_text, 
                                    clear_log_text, EvalKernel)
import os
from nose.plugins.attrib import attr


@attr('network')
def test_download_magic():
    kernel = get_kernel(EvalKernel)
    kernel.do_execute("%download --filename TEST.txt https://raw.githubusercontent.com/calysto/metakernel/master/LICENSE.txt")
    text = get_log_text(kernel)
    assert "Downloaded 'TEST.txt'" in text, text
    assert os.path.isfile("TEST.txt"), "File does not exist: TEST.txt"

    clear_log_text(kernel)

    kernel.do_execute("%download https://raw.githubusercontent.com/calysto/metakernel/master/LICENSE.txt")
    text = get_log_text(kernel)
    assert "Downloaded 'LICENSE.txt'" in text, text
    assert os.path.isfile("LICENSE.txt"), "File does not exist: LICENSE.txt"


def teardown():
    for fname in ['TEST.txt', 'LICENSE.txt']:
        try:
            os.remove(fname)
        except:
            pass



# Copyright (c) Metakernel Development Team.
# Distributed under the terms of the Modified BSD License.

from metakernel import Magic, option

class PipeMagic(Magic):
    def __init__(self, *args, **kwargs):
        super(PipeMagic, self).__init__(*args, **kwargs)

    def cell_pipe(self, pipe_str):
        """
        %%pipe FUNCTION1 | FUNCTION2 ...

        The pipe cell will "pipe" the contents of a cell
        through a series of function calls. All of the
        functions must be defined in the language, and
        the kernel must support the `do_function_direct`
        method.

        Example:
            %%pipe f1 | f2 | f3
            CELL CONTENTS

            is the same as issuing:

            f3(f2(f1("CELL CONTENTS")))
        """
        functions = [function.strip() for function in pipe_str.split("|")]
        self.retval = self.code
        for function in functions:
            self.retval = self.kernel.do_function_direct(function, self.retval)
        self.evaluate = False

    def post_process(self, retval):
        return self.retval

def register_magics(kernel):
    kernel.register_magics(PipeMagic)

def register_ipython_magics():
    from IPython.core.magic import register_cell_magic
    from IPython import get_ipython

    @register_cell_magic
    def pipe(line, cell):
        """
        """
        ip = get_ipython()
        functions = [function.strip() for function in line.split("|")]
        retval = cell
        for function in functions:
            f = eval(function, ip.ns_table["user_global"])
            retval = f(retval)
        return retval
